#!/usr/local/bin/perl
#  install.pl - the CBB install utility
#              This script installs CBB
#
#  Written by Curtis Olson.  Started October 11, 1994.
#
#  Copyright (C) 1994, 1995, 1996  Curtis L. Olson  - curt@sledge.mn.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: install.pl,v 2.12 1996/10/22 21:43:57 curt Exp $
# (Log is kept at end of this file)


require "common.pl";

# tcl execs
@tcl_execs = ("cbb", "conf-reports");

# perl binaries
@perl_execs = ("wrapper.pl");

# tcl includes
@tcl_incs = ("bindings.tk", "categories.tk", "common.tk", "file.tk", 
		"filebox.tk", "main.tk", "menu.tk");

# perl includes
@perl_incs = ("categories.pl", "common.pl", "engine.pl", "export.pl", 
		"file.pl", "import.pl", "log.pl", "memorized.pl", 
		"reports.pl");

# reports
@reports = ("ave-by-cat.pl", "by-cat.pl", "miss-check.pl", "shrt-by-cat.pl", 
	    "txn-list.pl");

@reports_conf = ("reports.conf");

# graphs
@perl_graphs = ("cat-col.pl", "cat-pie.pl", "cat2-col.pl", "txn-list.pl");

@tcl_graphs = ("graphbal", "graphcol", "graphcolpos", "graphpie");

@graphs_conf = ("graphs.conf");

# other files
@other_files = ("author.xbm", "cbb.xbm", "csh.src", "default.cat", 
		"extern.conf");

# contributed stuff
@perl_contrib = ("fetch-latest.pl", "invest.pl", "migrate-to-0.60a.pl", 
		 "recur.pl", "upgrade-splits.pl", "yearend.pl");

# possible browsers
@browsers = ("netscape", "Mosaic", "mosaic", "rmosaic", "glimpse",
	     "amaya", "chimera", "lynx");

# get home directory
@pw = getpwuid($<);

$HOME = $ENV{HOME}
     || $ENV{LOGDIR}
     || $pw[7]
     || die "No home directory";

$home_dir = &file_dirname($HOME);
$home_base = &file_basename($HOME);
 
# default paths
$def_perlpath = "";
@perl_poss = ("/usr/local/bin/perl", "/usr/bin/perl");

$def_wishpath = "";

@wish_poss = ("/usr/local/bin/wish4.1", "/usr/bin/wish4.1",
              "/usr/bin/X11/wish4.1", "/usr/local/bin/wish4.0",
              "/usr/bin/wish4.0", "/usr/bin/X11/wish4.0",
              "/usr/bin/wish", "/usr/bin/X11/wish",
              "/usr/local/bin/wish");

$def_binpath = &which("cbb");
  $pos = rindex($def_binpath, "/");
  $def_binpath = substr($def_binpath, 0, $pos);
  if ( $def_binpath eq "" ) {
        $def_binpath = "/usr/local/bin";
  }
  $def_binpath .= "/";

$def_libpath = "";
$def_version = "Version";
$def_icon = "cbb.xbm";
$def_man = "cbb-man";


# Ok, first lets find out the current version

open( VERSION, "$def_version" ) || die "Cannot open file:  $def_version\n";
$version = <VERSION>;
chop($version);

print "\nInstalling and Configuring CBB $version:\n\n";
print "Press ^C at any time to abort.\n\n";

print "    NOTE:  You may use ~ and ~user to specify a home directory as\n";
print "           part of an installation file name or path.\n\n";

# Now, make sure we have the prerequisites.

foreach $tmp (@perl_poss) {
    if ( (-e $tmp) && ($def_perlpath eq "") ) {
	$def_perlpath = $tmp;
    }
}

$perlpath = "";
while ( $perlpath eq "" ) {
    print "What is the name of your perl binary? [$def_perlpath] ";
    $perlpath = <STDIN>;
    chop($perlpath);

    if ( $perlpath eq "" ) {
	$perlpath = $def_perlpath;
    }

    # psuedo-glob
    if ( ($perlpath =~ m/~\//) || ($perlpath =~ m/~$/) ) {
        # ~/blah form
        $perlpath =~ s/~/$HOME/;
    } else {
        # ~user/blah from
        $perlpath =~ s/~/$home_dir/;
    }

    if ( ! -e $perlpath ) {
	$perlpath = "";
    }
}
print "    Using $perlpath\n\n";

foreach $tmp (@wish_poss) {
    if ( (-e $tmp) && ($def_wishpath eq "") ) {
	$def_wishpath = $tmp;
	break;
    }
}

$wishpath = "";
while ( $wishpath eq "" ) {
    print "What is the name of your wish (tk4.x) binary [$def_wishpath] ";
    $wishpath = <STDIN>;
    chop($wishpath);

    if ( $wishpath eq "" ) {
	$wishpath = $def_wishpath;
    }

    # psuedo-glob
    if ( ($wishpath =~ m/~\//) || ($wishpath =~ m/~$/) ) {
        # ~/blah form
        $wishpath =~ s/~/$HOME/;
    } else {
        # ~user/blah from
        $wishpath =~ s/~/$home_dir/;
    }

    if ( ! -e $wishpath ) {
	$wishpath = "";
    }
}
print "    Using $wishpath\n\n";


# check for a web browser

$def_browser = "no-browser";

print "Looking for web browsers: ";
foreach $browser (@browsers) {
    print "$browser ";
    $temp = &which($browser);
    $pos = rindex($temp, "/");
    $temp = substr($temp, 0, $pos);
    # print "'$temp'\n";
    if ( $temp ne "" ) {
	if ( $def_browser eq "no-browser" ) {
	    $def_browser = $browser;
	    # print "choosing $def_browser\n";
	    break;
	}
    }
}
print "\n";

if ( $def_browser eq "" ) {
    print "Warning:  cannot find a web browser.  CBB uses this for online help.\n"
} else {
    print "    Found $def_browser.  CBB will use this for online help.\n\n";
}


# check for gnuplot

print "Looking for gnuplot:\n";
$gnuplot = &which("gnuplot");
$pos = rindex($gnuplot, "/");
$gnuplot = substr($gnuplot, 0, $pos);
if ( $gnuplot =~ m/ / ) {
    print "    Warning:  cannot find gnuplot.  CBB uses this for graphing.\n\n";
} else {
    print "    Found gnuplot.  CBB uses this for graphing.\n\n";
}


# Now, where do we want to install things ...

$binpath = "";
while ( $binpath eq "" ) {
    print "Where should the binaries be installed? [$def_binpath] ";
    $binpath = <STDIN>;
    chop($binpath);

    if ( $binpath eq "" ) {
	$binpath = $def_binpath;
    }

    # psuedo-glob
    if ( ($binpath =~ m/~\//) || ($binpath =~ m/~$/) ) {
	# ~/blah form
	$binpath =~ s/~/$HOME/;
    } else {
	# ~user/blah from
	$binpath =~ s/~/$home_dir/;
    }

    if ( ! -e $binpath || ! -d $binpath ) {
	print "    $binpath does not exist.  Should I create it? [y] ";
	$result = <STDIN>;
	chop($result);
	if ( ($result =~ /^[Yy]/) || ($result eq "") ) {
	    $status = system( "mkdir $binpath" );
	    if ( $status ) {
		$code = $status / 256;
		print"\'mkdir $binpath\' failed with an exit code of ";
		print"$code.\n";
		exit( $code );
	    }
	    chmod(0755, $binpath);
	} else {
	    print "    Very well ... but you better do it yourself. ;)\n";
	}
    }

    if ( $binpath =~ m/wish$/ ) {
	print "    Warning:  CBB cannot run with wish, it requires bin.\n";
	print "    See the README file for more information.\n";
	$binpath = "";
    }
}
print "    Using $binpath\n\n";

# derive a default $libpath from $binpath
$tmp = $binpath;
if ( substr($tmp, length($tmp) - 1) eq "/" ) {
    chop($tmp);
}
$def_libpath = &file_dirname($tmp) . "lib/cbb/";

$libpath = "";
while ( $libpath eq "" ) {
    print "Where should the support files be installed? [$def_libpath] ";
    $libpath = <STDIN>;
    chop($libpath);

    if ( $libpath eq "" ) {
	$libpath = $def_libpath;
    }

    # psuedo-glob
    if ( ($libpath =~ m/~\//) || ($libpath =~ m/~$/) ) {
        # ~/blah form
        $libpath =~ s/~/$HOME/;
    } else {
        # ~user/blah from
        $libpath =~ s/~/$home_base/;
    }

    if ( ! -e $libpath || ! -d $libpath ) {
	print "    $libpath does not exist.  Should I create it? [y] ";
	$result = <STDIN>;
	chop($result);
	if ( ($result =~ /^[Yy]/) || ($result eq "") ) {
	    $status = system( "mkdir -p $libpath" );
	    if ( $status ) {
		$code = $status / 256;
		print"\'mkdir $libpath\' failed with an exit code of ";
		print"$code.\n";
		exit( $code );
	    }
	    chmod(0755, $libpath);
	} else {
	    print "    Very well ... but you better do it yourself. ;)\n";
	}
    }

    if ( $libpath =~ m/wish$/ ) {
	print "    Warning:  CBB cannot run with wish, it requires lib.\n";
	print "    See the README file for more information.\n";
	$libpath = "";
    }
}

# report path
$reportpath = "$libpath/reports";
if ( ! -e "$libpath/reports" || ! -d "$libpath/reports" ) {
    system( "mkdir $libpath/reports" );
    chmod(0755, "$libpath/reports");
}

# graph path
$graphpath = "$libpath/graphs";
if ( ! -e "$libpath/graphs" || ! -d "$libpath/graphs" ) {
    system( "mkdir $libpath/graphs" );
    chmod(0755, "$libpath/graphs");
}

# contrib path
$contribpath = "$libpath/contrib";
if ( ! -e "$libpath/contrib" || ! -d "$libpath/contrib" ) {
    system( "mkdir $libpath/contrib" );
    chmod(0755, "$libpath/contrib");
}

# cbb-man path
$cbbman_path = "$libpath/cbb-man";
if ( ! -e "$libpath/cbb-man" || ! -d "$libpath/cbb-man" ) {
    system( "mkdir $libpath/cbb-man" );
    chmod(0755, "$libpath/cbb-man");
}

# cbb-man icon path
$iconpath = "$cbbman_path/icons";
if ( ! -e "$libpath/cbb-man/icons" || ! -d "$libpath/cbb-man/icons" ) {
    system( "mkdir $libpath/cbb-man/icons" );
    chmod(0755, "$libpath/cbb-man/icons");
}


print "    Using $libpath\n";
print "    Using $libpath" . "reports/\n";
print "    Using $libpath" . "graphs/\n";
print "    Using $libpath" . "contrib/\n";
print "    Using $libpath" . "cbb-man/\n";
print "    Using $libpath" . "cbb-man/icons/\n";
print "\n";

# Now ... install CBB!

# install tcl executables

print ( "Now installing CBB:\n" );

&my_print_init();
&my_print("\nInstalling executables:  ");

foreach $file ( @tcl_execs ) {
    &install($file, $binpath, "tcl", 0755);
    &my_print("$file ");
}


# install perl executables

foreach $file (@perl_execs) {
    &install($file, $binpath, "perl", 0755);
    &my_print("$file ");
}


# install perl includes

&my_print_init();
&my_print("\n\nInstalling perl includes:  ");

foreach $file (@perl_incs) {
    &install($file, $libpath, "perl", 0644);
    &my_print("$file ");
}


# install reports

&my_print_init();
&my_print("\n\nInstalling report files:  ");

foreach $file (@reports_conf) {
    &install("reports/$file", $libpath, "text", 0644);
    &my_print("$file ");
}

foreach $file (@reports) {
    &install("reports/$file", $libpath, "perl", 0755);
    &my_print("$file ");
}


# install graphs

&my_print_init();
&my_print("\n\nInstalling graph files:  ");

foreach $file (@graphs_conf) {
    &install("graphs/$file", $libpath, "text", 0644);
    &my_print("$file ");
}

foreach $file (@perl_graphs) {
    &install("graphs/$file", $libpath, "perl", 0755);
    &my_print("$file ");
}

foreach $file (@tcl_graphs) {
    &install("graphs/$file", $libpath, "tcl", 0755);
    &my_print("$file ");
}


# install tcl includes

&my_print_init();
&my_print("\n\nInstalling tcl includes:  ");

foreach $file (@tcl_incs) {
    &install($file, $libpath, "tcl", 0644);
    &my_print("$file ");
}


# install other files

&my_print_init();
&my_print("\n\nInstalling other files:  ");

foreach $file (@other_files) {
    &install($file, $libpath, "text", 0644);
    &my_print("$file ");
}


&my_print_init();
&my_print("\n\nInstalling other contributed stuff:  ");

foreach $file (@perl_contrib) {
    &install("contrib/$file", $libpath, "perl", 0755);
    &my_print("$file ");
}


# install online man files

&my_print_init();
&my_print("\n\nInstalling CBB online manual:  ");

&install("cbb-man.txt", $libpath, "text", 0644);
&my_print("cbb-man.txt ");

foreach $file (`ls $def_man/*.html`) {
    $file = &file_basename($file);
    chop($file);

    &install("cbb-man/$file", $libpath, "html", 0644);
    &my_print("$file ");
}

foreach $file (`ls $def_man/*.gif`) {
    $file = &file_basename($file);
    chop($file);
    &my_print("$file ");

    system("cp ./$def_man/$file $libpath/cbb-man/$file");
    chmod(0644, "$libpath/cbb-man/$file") || 
	die "Cannot chmod $libpath/cbb-man/$file";
}


foreach $file (`ls $def_man/icons/*`) {
    $file = &file_basename($file);
    chop($file);
    &my_print("$file ");

    system("cp ./$def_man/icons/$file $libpath/cbb-man/icons/$file");
    chmod(0644, "$libpath/cbb-man/icons/$file") || 
	die "Cannot chmod $libpath/cbb-man/icons/$file";
}


print "\n\nInstallation successful!\n";
print "\n";
print "Make sure '$binpath' is in your \$PATH, \n";
print "then type 'cbb' and have fun!\n";
print "\n";
print "The CBB manual is available online via the 'Help' menu.\n";
print "\n";
print "Please send comments and suggestions to 'curt\@sledge.mn.org'\n";
print "Remember to visit the CBB web page at ";
print "'http://www.menet.umn.edu/~clolson/cbb/'\n";


# a function to duplicate the unix/shell command "which" since it may not
# always be available
sub which {
    local($file) = @_;
    local(@PATHS) = split(/:/, $ENV{PATH});

    foreach $path (@PATHS) {
	# print "Checking $path\n";
	if ( (-e "$path/$file") && ($path ne ".") ) {
	    return "$path/$file";
	}
    }

    return "";
}


sub install {
    local($file, $path, $type, $perms) = @_;

    # print ( "Creating $path/$file from ./$file\n" );

    open( IN, "<./$file" ) || die "Cannot open ./$file";
    open( OUT, ">$path/$file" ) || die "Cannot create $path/$file";

    if ( $type eq "tcl" ) {
	$junk = <IN>;	# ignore first line
	print( OUT "#!$wishpath -f\n" );
    } elsif ( $type eq "perl" ) {
	$junk = <IN>;	# ignore first line
	print( OUT "#!$perlpath\n" );
    }

    while ( <IN> ) {
	if ( $type eq "tcl" ) {
	    s/^ *set lib_path.*$/set lib_path \"$libpath\"/;
	    s/\"Version <not_installed>.*\"/\"$version\"/;
	    s/\s+exec netscape.*$/exec $def_browser \$lib_path\/cbb-man\/cbb-man.html \&/;
            # " a comment
	} elsif ( $type eq "perl" ) {
	    s/^ *\$cbb_incl_dir = .*$/\$cbb_incl_dir = \"$libpath\";/;
	    s/^ *\$reportpath = .*$/\$reportpath = \"$reportpath\";/;
	    s/^ *\$graphpath = .*$/\$graphpath = \"$graphpath\";/;
	    s/\"Version <not_installed>.*\"/\"$version\"/;
            # " a comment
	} elsif ( $type eq "html" ) {
	    s/\/usr\/local\/src\/www\/latex2html\/icons\//$iconpath\//g;
	}

	print( OUT $_ );
    }

    close( IN );
    close( OUT );

    chmod($perms, "$path/$file") || die "Cannot chmod $perms $path/$file";
}


sub my_print_init {
    $first_line = 1;
    $cur_len = 0;
}


sub my_print {
    local($line) = @_[0];

    if ( length($line) + $cur_len < 75 ) {
        print $line;
    } else {
	$cur_len = 0;
	if ( $first_line ) {
	    print "\n$line";
	} else {
	    print "\n    $line";
	}
    }

    $first_line = 0;
    $cur_len += length($line);
}


# ----------------------------------------------------------------------------
# $Log: install.pl,v $
# Revision 2.12  1996/10/22 21:43:57  curt
# Added a new report:  ave-by-cat.pl
#
# Revision 2.11  1996/10/04 19:48:24  curt
# Added a script to automatically fetch and install the latest version of CBB.
#
# Revision 2.10  1996/10/03 22:02:26  curt
# I found a way in perl to get the century directly, so I was able to
# eliminate the dependency on the external Unix date command.
#
# Revision 2.9  1996/10/03 04:13:40  curt
# Refined default century handling code.
#
# Revision 2.8  1996/10/01 17:21:08  curt
# Added a new missing check # report.
#
# Revision 2.7  1996/09/30 15:19:45  curt
# Added chimera to potential web browsers list.
#
# Revision 2.6  1996/09/27 21:43:41  curt
# Added a text version of the manual to the Help menu.
#
# Revision 2.5  1996/09/26 20:38:51  curt
# Default to tk4.1 if it is available.
# Added a couple more web browsers to the search list.
#
# Revision 2.4  1996/08/29 14:17:04  curt
# Reworded an initial install prompt message for better clarity.
#
# Revision 2.3  1996/07/24 20:11:24  curt
# Die on error opening version file.
#
# Revision 2.2  1996/07/13 02:57:47  curt
# Version 0.65
# Packing Changes
# Documenation changes
# Changes to handle a value in both debit and credit fields.
#
# Revision 2.1  1996/02/27  05:35:46  curt
# Just stumbling around a bit with cvs ... :-(
#
# Revision 2.0  1996/02/27  04:42:59  curt
# Initial 2.0 revision.  (See "Log" files for old history.)
